/*****************************************************************************************
* File name: C2EEPROM.c
* Purpose:  Copy Data from RAM to EEPROM.			 
*
* Description:	This Program Acknowledges the Reception of the Copy data from RAM to EEPROM 
*				and Updates the HLSW in SPU EEPROM. It Copies the HLSW from PRAM and/or DRAM
*               to EEPROM.
*****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif

#include "l_hwmap.h"
#include "spulib.h"

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

#ifndef SIMULATOR
extern u4 EEPROM_CopyPRAMToEEPROM(u4 src,u4 dest,u4 size);
extern u4 EEPROM_CopyDRAMToEEPROM(u4 src,u4 dest,u4 size);
#endif

/*================================================================================================*/

/*
This function handles the copy to eeprom command received from DPU
*/
void PerformCopyToEeprom(u4 buf[])
{
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
	u4 sid=buf[1]&0xFFFF;
	
	/*=============================================================================================*/
	
	if (sid!=5) /* Negative Acknowledgment:  Invalid Structure ID*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID;
	  	fifoTransmitMsg.Block1.NackParameter = sid;

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else if ((buf[3]!=1)&&(buf[3]!=0x11)&&(buf[3]!=0x12)) /* Invalid SID Parameter: Memory ID*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID_PARAMETER;
	  	fifoTransmitMsg.Block1.NackParameter = buf[3];

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else if(buf[5] < K_PM_START_EEPROM) /* Invalid SID Parameter: Invalid Address in EEPROM*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM| K_INVALID_SID_PARAMETER;
	  	fifoTransmitMsg.Block1.NackParameter = buf[5];

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else if((buf[5] + roldiv(buf[6]+2, 3)) > K_PM_END_EEPROM) /* Invalid SID Parameter: Length Exceeds EEPROM Address Range*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM| K_INVALID_SID_PARAMETER;
	  	fifoTransmitMsg.Block1.NackParameter = buf[6];

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else /*Positive Acknowledgment */		
	{
		/* Send the Positive Acknowledgment to DPU */
		fifoTransmitMsg.Header = K_PACK_PERFORM;
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg);
		KS_TaskSleep(100);

		/*Case of Program Memory as Source address*/ 
#ifndef SIMULATOR
		if(buf[3]==1)
		{
			/*Copy data from PRAM to EEPROM*/
			EEPROM_CopyPRAMToEEPROM(buf[4],buf[5],buf[6]);
		}
		else/*Case of Data Memory as Source address*/ 
		{
			/* Point to Source Address of Ext DRAM if Signaled in Parameter2*/
			if(buf[3]==0x12) 
			{
				buf[4] += K_DMADDR_BASE_EXTDRAM;
			}
			
			/*Copy data from DRAM to EEPROM*/
			EEPROM_CopyDRAMToEEPROM(buf[4],buf[5],buf[6]);
		}
#endif
	}
}
