/***************************************************************************************
* File name: asm_lib.s
* Version.Revision: HPSPU-HLSW-OB-V12R8
*
* Purpose:  Various auxiliary routines in assembler
*
* Public Functions:  
*
* Private Functions: 
*
* Description:	
*
* Creation date & author: 31.05.2006 by Roland Ottensamer
* Version, Update date & author: HPSPU-HLSW-OB-V12R3 31.05.2006 by Roland Ottensamer
****************************************************************************************/

/* The Header Files and the Routine Declaration
****************************************************************************************/

//#include <def21020.h>      /* STATUS REGISTER BIT DEFINITIONS FOR ADSP-21020 */
//#include "macro.h"         /* Macro's for System Kernel (Full Path)*/
	
/* 
	signed integer division for PACS 
	prototype: int roldivs (int x, int y)

	calculates z = x/y (registers: r0 = r4/r8)
	division by zero yields 0xffffffff for compatibility with the / division
	cycles: 22 + 16..111 (incl. prologue + epilogue), depending on input data
	clobbers: r0, r4, r8, r10
	uses/restores: r5
	calls: _roldiv
	
	author R. Ottensamer
	version date: 28.11.06	
*/
.segment/pm seg_pmco;
.global _roldivs, _roldiv;

#if 0
not validated !
_roldivs:
	dm(i7, m7) = r5;			/* save r5 */
	dm(i7, m7) = r9;			/* save r9 */

	r9 = lshift r4 by -31;		/* r9 = sign(r4) */
	r4 = pass r4;
	if lt r4 = -r4;				/* r4 = -r4 if r4 < 0 */

	r5 = lshift r8 by -31;		/* r5 = sign(r8) */
	r8 = pass r8;
	if lt r8 = -r8;				/* r8 = -r8 if r8 < 0 */

	/* ccall _roldiv */
	r2 = i6;					/* hold old frame */ 
	i6 = i7; 					/* swap stack and frame */
	jump _roldiv (db);			
	dm (i7, m7) = r2;			/* save old frame */
	dm (i7, m7) = pc;			/* save return address */

	comp(r9, r5);
	if eq jump _roldivs_ret;
	r0 = -r0;

_roldivs_ret:
	i12 = dm (m7, i6);			/* leaf_exit */
	r9 = dm (m6, i7);			/* restore r9 */
	r5 = dm (m6, i7);			/* restore r5 */
	
	jump (m14, i12) (db);			/* leaf_exit continued */
	i7 = i6;
	i6 = dm (m5, i6);	
#endif	
/*********************************************************************************/
/* 
	unsigned integer division
	for PACS by R. Ottensamer
	prototype: unsigned int roldiv (unsigned int x, unsigned int y)
	calculates z = x/y (registers: r0 = r4/r8)
	division by zero yields 0xffffffff for compatibility with the / division
	cycles: 16 - 111 (incl. prologue + epilogue), depending on input data
	clobbers: r0, r1, r2, r4, r8, r12
	version date: 1.2.06	
*/

_roldiv:
		r2 = leftz r4, r0 = m5; 						/* get x shift, r0 = 0 */
		r1 = leftz r8;									/* get y shift */
		r2 = r1 - r2, r1 = m6; 							/* calculate shift, r1 = 1 */
		if le jump (pc, xrdiv) (db);					/* if shift <= 0 go to lsb adjust */
		r8 = pass r8, r12 = m7;							/* if y==0, r12=0xffffffff */
		if eq r0 = r12 - r1, r2 = m6;					/* then z=fffffffe, sh=1 */

		r8 = lshift r8 by r2;							/* shift y to same magnitude */

		lcntr = r2, do dend until lce;					/* the shift and subtract loop */
		r12 = r4 - r8, r2 = curlcntr;					/* use the curlcntr to decrement r2 */
	 	if ac r0 = r0 or lshift r1 by r2, r4 = r12; 	/* AC, because GT is signed! */			
dend:	r8 = lshift r8 by -1; 							/* y >>= 1 */ 

xrdiv:	r12 = r4 - r8, i12 = dm (m7, i6); 				/* if x >= y ... , epilogue a la leaf_exit*/
		if ac r0 = r0 + 1;								/* ... then adjust lsb */
													
		jump (m14, i12) (db);							/* leaf_exit continued */
		i7 = i6;
		i6 = dm (m5, i6);
.endseg;

/*********************************************************************************/
/* 
	put n bits from 32 bit source word to destination bit stream (in 32bit memory)
 	for PACS by R. Ottensamer
	prototype: void putnbits (u4 value, u4 offset, u4 nbits, u4 *dest)


	N = r12 = nbits
	O = r8 = bit offset
	r4 = value

	clobbers: r0, r1, r2, r4, r8, r12, i4, m4
	version date: 7.2.06	
*/


.segment/pm seg_pmco;
.global _putnbits;


_putnbits:
		i4 = dm (1, i6);							/* read dest address off the stack */
		r0 = lshift r8 by -5;						/* WO = O/32 */
		m4 = r0;

		r1 = lshift r0 by 5;						/* LO = O-32*WO | last use of O */
		r1 = r8 - r1;									
		r2 = 32;
		r1 = r2 - r1;								/* N1 = 32-LO | last use of LO */

		r8 = r1 - r12, r2 = m7;						/* R = N1 - N */

		r2 = lshift r2 by r12;						/* apply N-mask on source */
		if lt jump (pc, brkn) (db);
		r2 = not r2;					
		r4 = r4 and r2;

whole:												/* needs R, N, SOURCE, DEST */
		r8 = r8 or lshift r12 by 6;					/* DEP = <R>:<N> | last use of R and N */
		r2 = m7;
		r2 = fdep r2 by r8;							/* MASK = ... */
		r2 = not r2;								/* invert */
		r1 = dm (m4, i4);							/* get copy of destination word */
		r1 = r1 and r2;								/* clear bits in dest copy */
		jump (pc, ptend) (db);
		r1 = r1 or fdep r4 by r8;					/* insert source bits */
		dm (m4, i4) = r1;							/* copy back */

brkn:												/* needs WO, N1, N, SOURCE, DEST */
ppart1:	
		r0 = m7;
		r2 = lshift r0 by r1;						/* MASK = ... by N1 */
		r8 = dm (m4, i4);							/* get copy of dest1 */
		r8 = r8 and r2;								/* clear bits in dest1 copy */

		r12 = r1 - r12;								/* -N2 = N1 - N, also needed in ppart2 */

		r8 = r8 or lshift r4 by r12;				/* insert first part of source bits */
		dm (m4, i4) = r8;
ppart2:
		r0 = m4;
		r0 = r0 + 1;
		m4 = r0;
		r0 = m7;
		r2 = lshift r0 by r12;						/* mask for second part */
		r8 = dm (m4, i4);							/* get copy of dest2 */
		r8 = r8 and r2;								/* clear bits in dest2 copy */
		r4 = rot r4 by r12;							/* rotate source */
		r2 = not r2;
		r4 = r4 and r2;								/* mask source */
		r8 = r8 or r4;								/* insert source in dest2 copy */
		dm (m4, i4) = r8;							/* copy back */

ptend:
 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);





/*********************************************************************************/
/* 
	put n bits from 32 bit source word to destination bit stream (in 32bit memory)
 	for PACS by R. Ottensamer
	prototype: void put_nbits_optimized (u4 value, u4 offset, u4 nbits, u4 *dest)


	N = r12 = nbits
	O = r8 = bit offset
	r4 = value

	clobbers: r0, r1, r2, r4, r8, r12, i4, m4
	version date: 7.2.06	
*/

.global _putnbitsOpt;

_putnbitsOpt:
		i4 = dm (1, i6);							/* read dest address off the stack */
		r0 = lshift r8 by -5;						/* WO = O/32 [WO = Word offset, O = bit offset]*/
		m4 = r0;

		r1 = fdep r8 by 0:5;						/* LO = local offset = bit offset % 32 */
		r2 = 32;
		r1 = r2 - r1;								/* N1 = 32-LO | last use of LO [N1 = local offset from lsb]*/

		r8 = r1 - r12;								/* R = N1 - N [R = number of remaining bits on the left of the destination word, N = number of bits to be deposit, m7 is always -1]*/

		if lt jump (pc, ptnbo_broken);					/* if R is < 0, the value must be deposit on 2 different target words */

ptnbo_whole:												/* needs R, N, SOURCE, DEST */
		r8 = r8 or lshift r12 by 6;					/* DEP = <R>:<N> | last use of R and N [build the parameter for the fdep function that is used below] */
		r2 = m7;
		r2 = fdep r2 by r8;							/* MASK = ...  [create a mask with ones on the destination area]*/
		r2 = not r2;								/* invert      [this mask is inverted (zeroes on the destination area)]*/
		r1 = dm (m4, i4);							/* get copy of destination word */
		r1 = r1 and r2;								/* clear bits in dest copy */
		jump (pc, ptnbo_ptend) (db);
		r1 = r1 or fdep r4 by r8;					/* insert source bits */
		dm (m4, i4) = r1;							/* copy back */

ptnbo_broken:												/* needs WO, N1, N, SOURCE, DEST */
ptnbo_ppart1:	
		r0 = m7;									/* r0 = -1 */
		r2 = lshift r0 by r1;						/* MASK = ... by N1 [r2 is now a mask of zeroes on the top part of the dest] */
		r8 = dm (m4, i4);							/* get copy of dest1 */
		r8 = r8 and r2;								/* clear bits in dest1 copy */

		r12 = r1 - r12;								/* -N2 = N1 - N, also needed in ppart2 [-N2 is the number of bits to deposit on the msb of the second dest word]*/

		r8 = r8 or lshift r4 by r12;				/* insert first part of source bits */
		dm (m4, i4) = r8;							/* copy back */
ptnbo_ppart2:
		modify(i4, m6);								/* m4, i4 is now pointing to the second destination word */
		r4 = rot r4 by r12;							/* rotate source */
		dm (m4, i4) = r4;							/* copy back */

ptnbo_ptend:
 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);

.endseg;

/*********************************************************************************/
/* 
	put 1 bit in destination bit stream (in 32bit memory)
 	for PACS by R. Ottensamer
	prototype: void putbit (u4 *dest, u4 offset, u4 value)
	note that value must be 0 or !=0
	cycles: 19 (incl. prologue + epilogue)
	clobbers: r0, r1, r2, r4, r8, i4, m4
	version date: 31.5.06	
*/


.segment/pm seg_pmco;
.global _putbit;

_putbit:
		r1 = lshift r8 by -5;						/* WO = O/32 */
		i4 = r4; 									/* i4 = &dest[offset] */
		m4 = r1;
		r2 = lshift r1 by 5;						/* LO = O-32*WO | last use of O */
		r1 = r8 - r2;									
		r0 = dm (m4, i4);							/* r0 = dest[offset] */
		r1 = r1 + 1;
		r0 = rot r0 by r1;
		r4 = 0xfffffffe;
		r0 = r0 and r4, r8 = m6;
		r12 = pass r12, r4 = m5;
		if ne r0 = r0 or r8;						/* if value != 0 */
		r1 = r4 - r1;
		r0 = rot r0 by r1;
		dm (m4, i4) = r0;							/* dest[offset] = modified 32 bit value */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);
/*---*/
		r1 = lshift r8 by -5;						/* WO = O/32 */
		i4 = r4; 									/* i4 = &dest[offset] */
		m4 = r1;
		r2 = lshift r1 by 5;						/* LO = O-32*WO | last use of O */
		r1 = r8 - r2;								/* last use of LO */	
		r8 = 31;
		r1 = r8 - r1, r2 = m6;						/* N1 = 32-LO */

		r0 = dm (m4, i4);							/* r0 = dest[offset] */
		r0 = bclr r0 by r1;

		r12 = pass r12;
		if ne r0 = bset r0 by r1;					/* if value != 0 */
		dm (m4, i4) = r0;							/* dest[offset] = modified 32 bit value */


.endseg;


/*********************************************************************************/
/* 
	get 1 byte from source buffer (in 32bit memory)
 	for PACS by R. Ottensamer
	prototype: int getbyte (u4 *src, u4 offset)
	cycles: 13 (incl. prologue + epilogue)
	clobbers: r0, r1, r2, r4, r8, i4, m4
	version date: 31.5.06	

	NEEDS TESTING	
*/

.segment/pm seg_pmco;
.global _getbyte;

_getbyte:
		r2 = lshift r8 by -2;						/* WO = O/4 */
		i4 = r4;									/* i4 = &src */
		r0 = lshift r2 by 2;				
		r1 = r0 - r2, m4 = r2;						/* LO = O - 4*WO */	
		r8 = 31;
		r1 = lshift r1 by 3;						/* r1 = r1*8 */
		r4 = r1 - r8, r2 = dm (m4, i4);				/* r4 is -24, -16, -8 or 0 | r2 = src[offset] */
		r0 = 0xff;
		r2 = lshift r2 by r4;						/* (r2 >> r4) */		
		r0 = r0 and r2;

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);

.endseg;


/*********************************************************************************/
/* 
	put 1 byte in destination buffer (in 32bit memory)
 	for PACS by R. Ottensamer
	prototype: void putbyte (u4 *dest, u4 offset, u4 value)
	cycles: 17 (incl. prologue + epilogue)
	clobbers: r0, r1, r2, r4, r8, i4, m4
	version date: 31.5.06	

	NEEDS TESTING
*/

.segment/pm seg_pmco;
.global _putbyte;

_putbyte:
		r2 = lshift r8 by -2;						/* word offset = offset/4 */
		i4 = r4;									/* i4 = &dest */
		r0 = lshift r2 by 2;						/* r0 ... truncated byte offset */
		r1 = r0 - r2, m4 = r2;						/* local byte offset r1 = o - 4*wo */	
		r8 = 31;
		r1 = lshift r1 by 3;						/* r1 = r1*8 */
		r4 = r8 - r1, r2 = dm (m4, i4);				/* r4 is 24, 16, 8 or 0 | r2 = dest[offset] */
		r1 = 0xff;
		r1 = lshift r1 by r4;						/* shape the mask */
		r1 = not r1;								/* invert */	
		r2 = r2 and r1;								/* apply on datum */ 
		r2 = r2 or lshift r0 by r4;					/* r2 | (value << r4) */		
		dm (m4, i4) = r2;							/* write back */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);

.endseg;


/*********************************************************************************/
/* 
	explode each word into 4 words (also works in place!), 1 byte each (in 32bit memory)
 	for PACS by R. Ottensamer
	prototype: void ro_explode (u4 *src, u4 *dest, u4 words)
	cycles: 16 + words*6 (incl. prologue + epilogue)
	clobbers: r0, r1, r2, r4, r8, r12, i4, i12
	version date: 1.6.06	
*/

.segment/pm seg_pmco;
.global _ro_explode;

_ro_explode:
		r0 = i5;									/* save i5 */
		dm (i7, m7) = r0;							

		r1 = r4 + r12;
		r1 = r1 - 1;								
		i4 = r1;									/* i4 = &src[words-1] */

		r2 = lshift r12 by 2;						/* words *4 */
		r2 = r8 + r2; 
		r2 = r2 - 1;
		i5 = r2;									/* i12 = &src[4*words -1] */ 

		lcntr = r12, do e_end until lce;			/* set up the loop counting src[words--] */
		r2 = dm (i4, m7);							/* get the data word and post-decrement i4 */
		r0 = fext r2 by 0:8;						/* fetch rightmost byte */
		r1 = fext r2 by 8:8, dm (i5, m7) = r0;		/* ... | put data byte and post-decrement i12 */
		r4 = fext r2 by 16:8, dm (i5, m7) = r1;		/* ... | ... */	
		r2 = fext r2 by 24:8, dm (i5, m7) = r4;		/* fetch leftmost byte | ... */
e_end:	dm (i5, m7) = r2;							/* put leftmost byte and post-decrement i12 */

		i5 = dm (m6, i7);							/* restore i5 */
		modify (i7, m6);							/* ... */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);
.endseg;


/*********************************************************************************
 	for PACS by A. Mazy
	prototype: void DeltaAsm (int *buf, int words)
    differences are made in place, from bottom to top, with
	16 bit sign extension
	
    for (i=words-1; i>0; i--)
	{ 
    	buf[i] = (buf[i] - buf[i-1]) &0xffff; 
		buf[i] = buf[i] > 0x7fff ? buf[i] | 0xffff0000 : buf[i];
	}


	r4 = buf
	r8 = words

	clobbers: r0, r1, r2, r4, r8, r12, i4, i12
	version date: 1.6.06	
*********************************************************************************/

.segment/pm seg_pmco;
.global _DeltaAsm;
_DeltaAsm:
		r8 = r8 - 1;
		r1 = r4 + r8;
		i4 = r1;									/* i4 = &buf[words-1] */

		r2 = dm(i4, m7);							/* r2 = buf[words-1] */
		r1 = 0xFFFF;								

		lcntr = r8, do Delta_end until lce;
		r4 = r2;									/* r4 = buf[i] */
		r2 = dm(i4, m7);							/* r2 = buf[i-1], i-- */
		r4 = r4 - r2;								/* r4 = buf[i] - buf[i-1] */
		r1 = r4 AND r1;								/* r4 = (buf[i] - buf[i-1]) & 0xFFFF */
		r4 = FDEP r4 by 0:16 (SE);					/* extend sign */
Delta_end:	dm(2, i4) = r4;								/* buf[i] = r4 */


 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);
.endseg;

/********************************************************************************
 	for PACS by A. Mazy
	prototype: void Put2ShortsIn1WordAsm(u4 *src, u4 *dest , u4 destSizeInWords)
	
	for(l=0;l<destSizeInWords;l++)
	{
	   dest[l] = (src[k]<<16)|(src[1+k]&0xFFFF);
	   k+=2;
	}


	r4 = src
	r8 = dest
	r12 = destSizeInWords

	clobbers: r0, r1, r2, r4, r8, r12, i4, i12
*********************************************************************************/

.segment/pm seg_pmco;
.global _Put2ShortsIn1WordAsm;
_Put2ShortsIn1WordAsm:
		r0 = i5;									/* save i5 */
		dm (i7, m7) = r0;							

		i5 = r8;									/* i5 = dest */
		i4 = r4;									/* i4 = src */
		r1 = 0xFFFF;

		lcntr = r12, do P2S1W_end until lce;
		r4 = dm(i4, m6);							/* r4 = src[k] */
		r8 = dm(i4, m6);							/* r8 = src[k+1] */
		r8 = r8 AND r1;
		r8 = r8 OR FDEP r4 by 16:16;				/* r8 = (src[k]<<16)|(src[1+k]&0xFFFF) */
P2S1W_end:	dm(i5, m6) = r8;						/* dest[l] = r8 */

		i5 = dm (m6, i7);							/* restore i5 */
		modify (i7, m6);							/* ... */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);
.endseg;

/********************************************************************************
 	for PACS by A. Mazy
	prototype: void Put2ShortsIn2Words(u4 *src, u4 *dest , u4 srcSizeInWords)
	
	for(i=0; i < srcSizeInWords;i++)
	{
		dest[k++] = src[i] >> 16;
		dest[k++] = src[i] & 0xFFFF;
	}	

	r4 = src
	r8 = dest
	r12 = srcSizeInWords
*********************************************************************************/

.segment/pm seg_pmco;
.global _Put2SIn2WAsm;
_Put2SIn2WAsm:
		r0 = i5;									/* save i5 */
		dm (i7, m7) = r0;							

		i5 = r8;									/* i5 = dest */
		i4 = r4;									/* i4 = src */
		r1 = 0xFFFF;

		lcntr = r12, do P2S2W_end until lce;
		r4 = dm(i4, m6);							/* r4 = src[i++] */
		r8 = LSHIFT r4 BY -16;						/* r8 = src[i] >> 16 */
		r4 = r4 AND r1, dm(i5, m6) = r8;			/* r4 = src[i] & 0xFFFF; dest[k++] = r8 */
P2S2W_end:	dm(i5, m6) = r4;						/* dest[k++] = r8 */

		i5 = dm (m6, i7);							/* restore i5 */
		modify (i7, m6);							/* ... */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);
.endseg;

/********************************************************************************
 	for PACS by A. Mazy
	prototype: void Put2ShortsIn2WordsSigned(u4 *src, u4 *dest , u4 srcSizeInWords)
	
	for(i=0; i < srcSizeInWords;i++)
	{
		dest[k++] = ((src[i]>>16)|((src[i]>>31)&0x1)*0xFFFF8000);
		dest[k++] = ((src[i]&0x7FFF)|(src[i]&0x8000)*0x1FFFF);
	}	

	r4 = src
	r8 = dest
	r12 = srcSizeInWords
*********************************************************************************/

.segment/pm seg_pmco;
.global _Put2SIn2WSignedAsm;
_Put2SIn2WSignedAsm:
		r0 = i5;									/* save i5 */
		dm (i7, m7) = r0;							

		i5 = r8;									/* i5 = dest */
		i4 = r4;									/* i4 = src */
		r1 = 0xFFFF;

		lcntr = r12, do P2S2WS_end until lce;
		r4 = dm(i4, m6);							/* r4 = src[i++] */
		r8 = ASHIFT r4 BY -16;						/* r8 = src[i] >> 16 (with sign extension)*/
		r4 = r4 AND r1, dm(i5, m6) = r8;			/* r4 = src[i] & 0xFFFF; dest[k++] = r8 */
		r4 = FEXT r4 by 0:16 (SE);					/* sign extend r4 */
P2S2WS_end:	dm(i5, m6) = r4;						/* dest[k++] = r8 */

		i5 = dm (m6, i7);							/* restore i5 */
		modify (i7, m6);							/* ... */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);
.endseg;

/********************************************************************************
 	for PACS by A. Mazy
	prototype: void MemCopyDmToDmAsm(u4 *src, u4 *dest , u4 srcSizeInWords)
	
	for(i=0; i<sizeInWords; i++) 
	{
		dest[i] = src[i];
	}

	r4 = src
	r8 = dest
	r12 = srcSizeInWords
*********************************************************************************/

.segment/pm seg_pmco;
.global _MemCopyDmToDmAsm;
_MemCopyDmToDmAsm:
		r0 = i5;									/* save i5 */
		dm (i7, m7) = r0;							

		i5 = r8;									/* i5 = dest */
		i4 = r4;									/* i4 = src */

		lcntr = r12, do memcpy_end until lce;
		r4 = dm(i4, m6);							/* r4 = src[i++] */
memcpy_end:	dm(i5, m6) = r4;						/* dest[k++] = r4 */

		i5 = dm (m6, i7);							/* restore i5 */
		modify (i7, m6);							/* ... */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);
.endseg;

/********************************************************************************
 	for PACS by A. Mazy
	prototype: void ExpandSign(int *buf, int sizeInWords);
	
    for (i=0; i<sizeInWords; i++)
		buf[i] = buf[i] > 0x7fff ? buf[i] | 0xffff0000 : buf[i];

	r4 = buf
	r8 = sizeInWords
*********************************************************************************/

.segment/pm seg_pmco;
.global _ExpandSignAsm;
_ExpandSignAsm:

		i4 = r4;									/* i4 = src */

		lcntr = r8, do expand_end until lce;
		r4 = dm(i4, m5);							/* r4 = buf[i] */
		r4 = FEXT r4 by 0:16 (SE);					/* extend sign of r4 */
expand_end:	dm(i4, m6) = r4;						/* buf[i++] = r4 */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);
.endseg;

/********************************************************************************
 	for PACS by A. Mazy
	prototype: void MakeDataSigned(int *buf, int sizeInWords);
	
	for (i=0; i<sizeInWords; i++)
		buf[i] = buf[i] - 0x8000;

	r4 = buf
	r8 = sizeInWords
*********************************************************************************/

.segment/pm seg_pmco;
.global _MakeDataSignedAsm;
_MakeDataSignedAsm:

		i4 = r4;									/* i4 = src */
		r1 = 0x8000;

		lcntr = r8, do makeSigned_end until lce;
		r4 = dm(i4, m5);							/* r4 = buf[i] */
		r4 = r4 - r1;								/* r4 = r4 - 0x8000 */
makeSigned_end:	dm(i4, m6) = r4;					/* buf[i++] = r4 */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);
.endseg;

/********************************************************************************
 	for PACS by A. Mazy
	prototype: void _SelectDetectors(u4* src, u4* dest, u4* table, u4 nbPixelsPerFrame, u4 nbFrames)

	u4 srcSize = nbPixelsPerFrame * nbFrames;

	for(pixel=0; pixel < nbPixelsPerFrame; pixel++) 
	{
   		// If the pixel is valid, write it
		if (table[pixel] != 0) 
		{
			for(frame = pixel; frame < srcSize; frame += nbPixelsPerFrame) 
			{
				dest[i++] = src[frame];
			}
		}
	}


	r4 = src					(i5)
	r8 = dest					(i4)
	r12 = table					(i3)
	stack1 = nbPixelsPerFrame	(r0)
	stack2 = nbFrames			(r1)
*********************************************************************************/

.segment/pm seg_pmco;
.global _SelectDetectorsAsm;
_SelectDetectorsAsm:

		r0 = dm (1, i6);							/* r0 = nbPixelsPerFrame */
		r1 = dm (2, i6);							/* r1 = nbFrames */

		r2 = i5;									/* save i5 */
		dm (i7, m7) = r2;							
		i5 = r4;									/* i5 = src */
		r2 = r4;									/* r2 = src */
		
		r4 = i3;
		dm (i7, m7) = r4;							/* save i3 */

		i4 = r8;									/* i4 = dest */
		i3 = r12;									/* i3 = table */
		r8 = -1;

		lcntr = r0, do select_pixloop_end until lce;/*loop on pixels*/
		r4 = dm(i3, m6);							/* r4 = table[pixel++] */
		r4 = pass r4;								/* test if r4 == 0 */
		if eq jump select_pixloop_end (db);
		r8 = r8 + 1;								
		m4 = r0;									/* m4 = nbPixelsPerFrame */
		
		r4 = r2 + r8;
		i5 = r4;									/* frame = pixel */
		lcntr = r1, do select_frameloop_end until lce; /*loop on frames*/
		r4 = dm(m5, i5);							/* r4 = src[frame] */
		dm(i4, m6) = r4;							/* dest[i++] = src[frame] */
select_frameloop_end: modify(i5, m4);				/* frame += nbPixelsPerFrame */
select_pixloop_end:	nop;

		i3 = dm (m6, i7);							/* restore i3 */
		modify (i7, m6);							/* ... */

		i5 = dm (m6, i7);							/* restore i5 */
		modify (i7, m6);							/* ... */

 		i12 = dm (m7, i6);							/* leaf_exit */
		jump (m14, i12) (db);							
		i7 = i6;
		i6 = dm (m5, i6);

.endseg;