/******************************************************************************************
* File name: Bol1_2.c
* Version.Revision: HPSPU-HLSW-OB-V12R1
*
* Purpose: Performs Data Compression for Two Photometric Modes.
*
* Public Functions: DMCH_cp.c
*					Dxs.c
*					p_proc.c
*					Average.c
*					integ.c
*					T_S_Red_Phot.c
*					Llc.c
*					Fill_out.c
*
* Private Functions: None
*
* Description:	This Program Performs Data Compression in Photometry for Bol0, Bol1 or Bol2
*				Modes Depending on the Received Compression Parameter.
*  
*				Bol0 is the Half Compression Mode
*				Bol1 is the Default Compression Mode
*				Bol2 Double the Compression Mode
*				
*
* Creation date & author: 27.07.2005 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 17.01.2006 by Ahmed Nabil Belbachir
******************************************************************************************/

/* The Header Files 
===================================================================================================*/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"	

/*================================================================================================*/
/*Declaration of the Used Functions
==================================================================================================*/

extern void Average(u4 Nb_Sel_Det);
extern void integ(u4 Number, u4 Nb_Detectors);
extern void integ_4_signed_int(u4 Number, u4 Nb_Detectors);
extern void T_S_Red_Phot();
extern void Llc_Phot();

/*================================================================================================*/
/*Declaration of the Used Global Variables
==================================================================================================*/

extern u4 Nb_Detectors_Bol_Blue; /* Number of Selected Detectors in Blue Photometry */


/*================================================================================================*/
/* The Main Program
==================================================================================================*/

void Bol1_2(u4 Number, u4 nbFrames, COMPRESSION_MODE mode)
{

	/*=============================================================================================*/
	/* Call Sequence of Compression Tasks
	===============================================================================================*/

	/* DMC Header Extraction and Compression */
	 DmcHeaderCompression(nbFrames, K_PHOTO_HEADER_SIZE, mode);
   
	if(mode.DataBlockId!=5) /* Blue Bolometers */
	{
		/* Be Sure That Number of RC is Compatible with Number of DX*/
		if (RawChannelSelectionPhoto.NbSelectedRawChannels > DxsPhotoBlue.NbSelectedPixels) 
			RawChannelSelectionPhoto.NbSelectedRawChannels = DxsPhotoBlue.NbSelectedPixels;

		/* Perform the Compression of the Science data for the 4 Sub-Arrays*/
		/* Detector Selection */

		if(Number==3)
			Dxs(nbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY, &DxsPhotoBlue, nbFrames, K_CBUF_SCIENCE_BLUE_PHOT1,4, mode);
		else
			Dxs(nbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY, &DxsPhotoBlue, nbFrames, K_CBUF_SCIENCE_BLUE_PHOT1,0, mode);

		/* preprocessing of the data */
		PreProcessing(mode.DetectorType);
	
		/* Robust Averaging and Glitch Rejection  */
		if(Number!=3)
			Average(nbFrames);
		
		/* Integration of 2 Averages to Double the Compression Ratio, If Required*/
		if(Number==2)
		{
			if (mode.BolcDataType == 0)	/*DataType = S-R*/
				/* Integration for Signed Int*/
				integ_4_signed_int(Number, DxsPhotoBlue.NbSelectedPixels);
			else
				/* Integration for Unsigned Int*/
				integ(Number, DxsPhotoBlue.NbSelectedPixels);
		}
		/* Temporal and Spatial Redundancy Reduction */
		T_S_Red_Phot();

		/* Lossless Coding */
		if (CompressionSoftwareRunning == K_YES) //check that the compression software has not been stopped in the meanwhile
		{
			Llc_Phot();

			/* Write the Compressed Data in the Output Buffer */
			BuildCompressedEntity(mode);
		}
	}
	else /* Red Bolometers */
	{
		/* Be Sure That Number of RC is Compatible with Number of DX*/
		if (RawChannelSelectionPhoto.NbSelectedRawChannels > DxsPhotoRed.NbSelectedPixels) 
			RawChannelSelectionPhoto.NbSelectedRawChannels = DxsPhotoRed.NbSelectedPixels;

		/* Detector Selection */
		Dxs(nbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY, &DxsPhotoRed, nbFrames, K_CBUF_SCIENCE_RED_PHOT, 0, mode);

		/* preprocessing of the data */
 		PreProcessing(mode.DetectorType);
	
		/* Robust Averaging and Glitch Rejection  */
		if(Number!=3)
			Average(nbFrames);

		/* Integration of 2 Averages to Double the Compression Ratio, If Required */
		if(Number==2)
		{
			if (mode.BolcDataType == 0)	/*DataType = S-R*/
				/* Integration for Signed Int*/
				integ_4_signed_int(Number , DxsPhotoRed.NbSelectedPixels);
			else
				/* Integration for Unsigned Int*/
				integ(Number, DxsPhotoRed.NbSelectedPixels);
		}
		/* Temporal and Spatial Redundancy Reduction */
		T_S_Red_Phot();

		/* Lossless Coding */
		if (CompressionSoftwareRunning == K_YES)
		{
			Llc_Phot();

			/* Write the Compressed Data in the Output Buffer*/
			BuildCompressedEntity(mode);
		}
	}
}
/* End of the Main Program 
===================================================================================================*/
