/*****************************************************************************************
* File name: Bol4.c
* Purpose: Performs Data Compression for the 4th Photometric Mode.
*
* Description:	  	This Program Performs Data Compression in Photometry for the
*					PhotoTransparentMode According to the Received Compression Parameter.
*****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"	

/*
In transparent mode, data are not compressed (DMC header is compressed).
There is a limitation on the number of selected pixels
*/
void PhotoTransparentMode(u4 nbFrames, COMPRESSION_MODE mode)
{
	/*=============================================================================================*/
	/* Reset of the HK Parameters of Compression Information. These HK are Only used While Reduction
	===============================================================================================*/

	Result.NbMaintainedRamps = 0;
	Result.GlitchCounter = 0;
	Result.SaturationFlag=0;
	Result.NbIntegratedRamps = 0;

	/*=============================================================================================*/
	/* Call Sequence of Compression Tasks
	===============================================================================================*/

	/* DMC Header Extraction and Compression */
	DmcHeaderCompression(nbFrames, K_PHOTO_HEADER_SIZE, mode);
   
	if(mode.DataBlockId!=5) /* Blue Bolometers */
	{
		/* Perform the Compression for 4 Sub-Arrays*/
		/* Detector Selection */
		Dxs(nbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY, &DxsPhotoBlue, nbFrames, K_CBUF_SCIENCE_BLUE_PHOT1, 0, mode);

		CompressedDataBuffer.Data = ProcBuff + 1;
		CompressedDataBuffer.SizeInWords = ProcBuff[0];

		/* Write the Compressed Data in the Output Buffer */
		BuildCompressedEntity(mode);
	}
	else /* Red Bolometers */
	{
		/* Detector Selection */
		Dxs(nbFrames * K_NB_PIXELS_IN_PHOTO_ARRAY, &DxsPhotoRed, nbFrames, K_CBUF_SCIENCE_RED_PHOT, 0, mode);
		
		CompressedDataBuffer.Data = ProcBuff + 1;
		CompressedDataBuffer.SizeInWords = ProcBuff[0];

		/* Write the Compressed Data in the Output Buffer.*/
		BuildCompressedEntity(mode);
	}
}
