/*****************************************************************************************
* File name: BUF_Bol.c
* Purpose: Performs Processing for the Buffer Transmission Mode in Photometry.
*
* Description:	This Program Performs Data Compression in Photometry for the
*				BUF Mode According to the Received Compression Parameter. It 
*				Fills the Memory for 15/60 second for Respectively SWL/LWL SPU with Data,
*				then Sends them in Tranparent Mode.
*
*				BUF is the Buffer Transmission Mode
*****************************************************************************************/
																			  
#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

#ifndef SIMULATOR
extern void MEM_CopyPMToDM(u4 src, u4 *dest, u4 size);
extern void MEM_CopyDMToPM(u4 *src, u4 dest, u4 size);
#endif

#ifdef SIMULATOR
extern u4 PramBuffer[0x200000];
#endif

/*=================================================================================================*/

void PhotoBufferTransmissionMode(COMPRESSION_MODE mode)
{
	u4 i=0;
	u4 firstCompressedEntityPacketIndex;
	u4 photoScienceBufferSizeIn32BitWords = K_PHOTO_DATA_3SEC_BUF_SIZE_IN_WORDS*4;
	u4 dataSizeIn16BitWords = K_PHOTO_DATA_3SEC_BUF_SIZE_IN_WORDS*2; 
	u4 dataSizeIn48BitWords = roldiv((K_PHOTO_DATA_3SEC_BUF_SIZE_IN_WORDS*2), 3);

    u4 bufferSizeBlueIn48BitWords = roldiv(((K_PHOTO_BLUE_HEADER_3SEC_BUF_SIZE_IN_WORDS+photoScienceBufferSizeIn32BitWords)*2), 3); 
    u4 bufferSizeBlueIn32BitWords = K_PHOTO_BLUE_HEADER_3SEC_BUF_SIZE_IN_WORDS+photoScienceBufferSizeIn32BitWords; 
	u4 bufferSizeRedIn48BitWords = roldiv(((K_PHOTO_RED_HEADER_3SEC_BUF_SIZE_IN_WORDS+photoScienceBufferSizeIn32BitWords)*2), 3); 
    u4 bufferSizeRedIn32BitWords = K_PHOTO_RED_HEADER_3SEC_BUF_SIZE_IN_WORDS+photoScienceBufferSizeIn32BitWords; 
#ifdef SIMULATOR
	u4 dmcBlueHeaderBufferSizeIn32BitWords = K_PHOTO_BLUE_HEADER_3SEC_BUF_SIZE_IN_WORDS;
	u4 dataSizeIn32BitWords = K_PHOTO_DATA_3SEC_BUF_SIZE_IN_WORDS*4;
#endif
	u4 dmcBlueHeaderBufferSizeIn16BitWords = K_PHOTO_BLUE_HEADER_3SEC_BUF_SIZE_IN_WORDS*2; 
	u4 dmcBlueHeaderBufferSizeIn48BitWords = roldiv(dmcBlueHeaderBufferSizeIn16BitWords, 3); 

	u4 dmcRedHeaderBufferSizeIn16BitWords = K_PHOTO_RED_HEADER_3SEC_BUF_SIZE_IN_WORDS*2; 
	u4 dmcRedHeaderBufferSizeIn48BitWords = roldiv(dmcRedHeaderBufferSizeIn16BitWords, 3); 

	/*=============================================================================================*/
	/* Reset of the HK Parameters of Compression Information. These HK are Only used While Reduction
	===============================================================================================*/
	
	Result.NbMaintainedRamps = 0;
	Result.GlitchCounter = 0;
	Result.SaturationFlag=0;
	Result.NbIntegratedRamps = 0;

	/*No Raw Channel Data are Sent in Transparent Mode */
	RawChannels.SizeInWords = 0;

	/*=============================================================================================*/
	
	if(mode.DataBlockId!=5) /* Blue Bolometers */
	{
		/* Store 4 Buffers Data in Memory */
		if(BufferTransmissionModeBufferId < 4)
		{
			/* DMC Header Extraction and Copy to PramBuffer */
			ReadInCircularBuffer(K_CBUF_HEADER, ProcBuff, K_PHOTO_BLUE_HEADER_3SEC_BUF_SIZE_IN_WORDS);

#ifndef SIMULATOR
			MEM_CopyDMToPM(ProcBuff, PramBuffer + BufferTransmissionModeBufferId * bufferSizeBlueIn48BitWords, dmcBlueHeaderBufferSizeIn16BitWords);
#else
			MemCopyDmToDm(ProcBuff, PramBuffer + BufferTransmissionModeBufferId * bufferSizeBlueIn32BitWords, dmcBlueHeaderBufferSizeIn32BitWords);
#endif
			/* Extract Science Data for The 4 Sub-regions (Sub-Arrays)*/
			for(i=0;i<4;i++) 
			{
				/* Science Data Extraction and Copy to PramBuffer */
				ReadInCircularBuffer(K_CBUF_SCIENCE_BLUE_PHOT1 + i, ProcBuff, K_PHOTO_DATA_3SEC_BUF_SIZE_IN_WORDS);
#ifndef SIMULATOR
			    MEM_CopyDMToPM(ProcBuff, PramBuffer + BufferTransmissionModeBufferId * bufferSizeBlueIn48BitWords + dmcBlueHeaderBufferSizeIn48BitWords + i* dataSizeIn48BitWords, dataSizeIn16BitWords);
#else
			    MemCopyDmToDm(ProcBuff,(PramBuffer + BufferTransmissionModeBufferId * bufferSizeBlueIn32BitWords + dmcBlueHeaderBufferSizeIn32BitWords + i* dataSizeIn48BitWords), K_PHOTO_DATA_3SEC_BUF_SIZE_IN_WORDS);
#endif
			}
			BufferTransmissionModeBufferId++;
		}
		/* Allow to fill the Last Buffer (5th) */
		else if (BufferTransmissionModeBufferId == 4)
		{
			/* Last DMC Header Extraction */
			ReadInCircularBuffer(K_CBUF_HEADER, ProcBuff+1, K_PHOTO_BLUE_HEADER_3SEC_BUF_SIZE_IN_WORDS);

			/* Extract Science Data from The 4 Sub-regions for the Last Buffer */
			for(i=0;i<4;i++) 
			{
				/* Last Buffered Science Data Extraction */
				ReadInCircularBuffer(K_CBUF_SCIENCE_BLUE_PHOT1 + i, ProcBuff+ (i* K_PHOTO_DATA_3SEC_BUF_SIZE_IN_WORDS)+K_PHOTO_BLUE_HEADER_3SEC_BUF_SIZE_IN_WORDS+1, K_PHOTO_DATA_3SEC_BUF_SIZE_IN_WORDS);
			}
			CompressedDataBuffer.Data = ProcBuff + 1;
			CompressedDataBuffer.SizeInWords = bufferSizeBlueIn32BitWords;

			BuildCompressedEntity(mode);
			firstCompressedEntityPacketIndex = mode.PacketIndex;
			
			/* Write the Compressed Data in the Output Buffer*/
		    for(i=0;i<4;i++)
			{
				if (BufferTransmissionModeEnabled == K_YES) // if the buffer transmission has not been interrupted in the meantime
				{

#ifndef SIMULATOR
					MEM_CopyPMToDM(PramBuffer + (i*bufferSizeBlueIn48BitWords) ,ProcBuff+1, bufferSizeBlueIn48BitWords*3);
#else
					MemCopyDmToDm(PramBuffer + (i*bufferSizeBlueIn32BitWords) ,ProcBuff+1, bufferSizeBlueIn32BitWords);
#endif
					mode.PacketIndex = firstCompressedEntityPacketIndex - 4 + i; //the packet index must be re-generated
					BuildCompressedEntity(mode);
				}
				else
					break;
			}

			/* Get ready for next buffer transmission mode*/
			BufferTransmissionModeBufferId = 0;
			BufferTransmissionModeEnabled = K_NO;
		}
	}
	else /* Red Bolometers */
	{
		/* Store 4 Buffers Data in Memory */
   		if(BufferTransmissionModeBufferId < 4)
		{
			/* DMC Header Extraction and Copy to PramBuffer */
			ReadInCircularBuffer(K_CBUF_HEADER, ProcBuff, K_PHOTO_RED_HEADER_3SEC_BUF_SIZE_IN_WORDS);

#ifndef SIMULATOR
		    MEM_CopyDMToPM(ProcBuff, PramBuffer + BufferTransmissionModeBufferId*bufferSizeRedIn48BitWords, dmcRedHeaderBufferSizeIn16BitWords);
#else
		    MemCopyDmToDm(ProcBuff, PramBuffer + BufferTransmissionModeBufferId*bufferSizeRedIn32BitWords, dmcBlueHeaderBufferSizeIn32BitWords*4);
#endif
			/* Science Data Extraction and Copy to PramBuffer */
			ReadInCircularBuffer(K_CBUF_SCIENCE_RED_PHOT, ProcBuff, photoScienceBufferSizeIn32BitWords);

#ifndef SIMULATOR
		    MEM_CopyDMToPM(ProcBuff,(PramBuffer + BufferTransmissionModeBufferId*bufferSizeRedIn48BitWords +dmcRedHeaderBufferSizeIn48BitWords), (dataSizeIn16BitWords*4));
#else
			MemCopyDmToDm(ProcBuff,(PramBuffer + BufferTransmissionModeBufferId*bufferSizeRedIn32BitWords +(dmcBlueHeaderBufferSizeIn32BitWords*4)), dataSizeIn32BitWords);
#endif
			BufferTransmissionModeBufferId++;
		}
		/* Allow to fill the Last Buffer (5th) */
		else if (BufferTransmissionModeBufferId ==4) 
		{
			/* Stop the ASW and Transmit the Buffered Data */
			BufferTransmissionModeEnabled =K_YES;
			
			/* Last DMC Header Extraction */
			ReadInCircularBuffer(K_CBUF_HEADER, ProcBuff+1, K_PHOTO_RED_HEADER_3SEC_BUF_SIZE_IN_WORDS);

			/* Last Data Science Data Extraction */
			ReadInCircularBuffer(K_CBUF_SCIENCE_RED_PHOT, ProcBuff + K_PHOTO_RED_HEADER_3SEC_BUF_SIZE_IN_WORDS + 1, photoScienceBufferSizeIn32BitWords);

			CompressedDataBuffer.Data = ProcBuff + 1;
			CompressedDataBuffer.SizeInWords = bufferSizeRedIn32BitWords;

			BuildCompressedEntity(mode);
			firstCompressedEntityPacketIndex = mode.PacketIndex;

			/* Write the Compressed Data in the Output Buffer*/
		    for(i=0;i<4;i++)
			{
				if (BufferTransmissionModeEnabled == K_YES) // if the buffer transmission has not been interrupted in the meantime
				{
#ifndef SIMULATOR
					MEM_CopyPMToDM(PramBuffer + (i*bufferSizeRedIn48BitWords) ,ProcBuff+1, bufferSizeRedIn48BitWords*3);
#else
					MemCopyDmToDm(PramBuffer + (i*bufferSizeRedIn32BitWords) ,ProcBuff+1, bufferSizeRedIn32BitWords);
#endif

					mode.PacketIndex = firstCompressedEntityPacketIndex - 4 + i;  //the packet index must be re-generated
					BuildCompressedEntity(mode);
				}
				else
					break;
			}				

			/* Get ready for next buffer transmission mode*/
			BufferTransmissionModeBufferId = 0;
			BufferTransmissionModeEnabled = K_NO;
		}	
	}
}
