/***************************************************************************************
* File name: integ.c
* Purpose:  Integrates Over Predefined Samples.
*
* Description:	The Integration Program is Used to Achieve the Additional
*				Compression Ratio to Fulfill the Telemetry Requirements.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"

/*================================================================================================*/
/*Declaration of the Used Functions
==================================================================================================*/

void integ(u4 Number, u4 Nb_Detectors);
void integ_4_signed_int(u4 Number, u4 Nb_Detectors);

/*=================================================================================================*/
/*Declaration of the Used Global Variables
===================================================================================================*/

extern u4 ChopperIsInTheSameChopperPlateau[130];

/*=================================================================================================*/
/* The Main Program 
===================================================================================================*/

void integ(u4 Number, u4 Nb_Detectors)
{
   	/* Declaration of Local Variables
	===============================================================================================*/

	u4 i,j,Summ=0, *y, Frames, factor=0, sizing=0, newsizing=0;
	
	/*=============================================================================================*/
	
	/* Show in The HK the Number of Integrated Ramps/Averages*/
	Result.NbIntegratedRamps = Number;

	/* Point to the Buffered Data */
	y=ProcBuff+1;
	Frames = roldiv(ProcBuff[0], Nb_Detectors);
	
	/*Case of Photometry. Processus is the Same with or without DEC/MEC Sequences */
	for(i=0;i<Nb_Detectors;i++)
	{
		Summ=0;
		factor=0;
		for(j=0;j<Frames;j++)
		{
			if(ChopperIsInTheSameChopperPlateau[j] == K_YES)
			{
				Summ += y[sizing++];
				factor++;
			}
			else
			{
				/*Integrating Over "factor" Number of Valid Samples */
				if (factor!=0)
					y[newsizing++] = roldiv(Summ, factor); 
				factor=1;
				Summ=0;
				Summ += y[sizing++];
			}
			if (factor==Number)
			{
				/*Integrating Over "factor" Number of Valid Samples */
				if (factor!=0)
					y[newsizing++] = roldiv(Summ, factor); 
				factor=0;
				Summ=0;
			}
		}
		/*Integrating Over the Remaining "factor" Number of Valid Samples */
		if (factor!=0)
			y[newsizing++] = roldiv(Summ, factor); 
	}
	/* Set the New Size after the Integration */
	ProcBuff[0] =newsizing;
}

void integ_4_signed_int(u4 Number, u4 Nb_Detectors)
{
   	/* Declaration of Local Variables
	===============================================================================================*/

	u4 i,j, Frames, factor=0, sizing=0, newsizing=0;
	int Summ=0, *y;
	/*=============================================================================================*/
	
	/* Show in The HK the Number of Integrated Ramps/Averages*/
	Result.NbIntegratedRamps = Number;

	/* Point to the Buffered Data */
	y=(int *)(ProcBuff+1);
	Frames = roldiv(ProcBuff[0], Nb_Detectors);
	
	/*Case of Photometry. Processus is the Same with or without DEC/MEC Sequences */
	for(i=0;i<Nb_Detectors;i++)
	{
		Summ=0;
		factor=0;
		for(j=0;j<Frames;j++)
		{
			if(ChopperIsInTheSameChopperPlateau[j] == K_YES)
			{
				Summ += y[sizing++];
				factor++;
			}
			else
			{
				/*Integrating Over "factor" Number of Valid Samples */
				if (factor!=0)
					y[newsizing++] = roldiv(Summ, factor); 
				factor=1;
				Summ=0;
				Summ += y[sizing++];
			}
			if (factor==Number)
			{
				/*Integrating Over "factor" Number of Valid Samples */
				if (factor!=0)
					y[newsizing++] = roldiv(Summ, factor); 
				factor=0;
				Summ=0;
			}
		}
		/*Integrating Over the Remaining "factor" Number of Valid Samples */
		if (factor!=0)
			y[newsizing++] = roldiv(Summ, factor); 
	}
	/* Set the New Size after the Integration */
	ProcBuff[0] =newsizing;
}
