/*****************************************************************************
 *
 * Project name: 			Plank LFI REBA / Herschel PACS SPU
 * Product name: 			LLSW_DRV
 * Object name: 			l_errcod
 * Filename: 				l_errcod.h
 * Language: 				C (ADSP-21020)
 * Compiler: 				G21K - r3.3
 * Company: 				CRISA
 * Author: 					Jaime Gomez/F. Torrero
 * Version: 				2
 * Creation date: 			17/Aug/2001
 * Last Modification date: 	23/09/2003 10:26:16
 *
 * Description ===============================================================
 *
 * This module defines the error codes returned by functions throughout the project
 *
 * Change log ===============================================================
 *
 * | DATE 	  | NEW VERSION 	| AUTHOR 	  | 	REASON FOR CHANGE 	|
 * ===========================================================================
 *
 * 17/Aug/01 	1			 	 F.Torrero		Creation
 *
 * 03/Sep/03	2				 F. Torrero		FPL-SMR-LLSW-002-CRS
 *												Error constant removed
 * ****************************************************************************/ 


#ifndef L_ERRCOD_H
#define L_ERRCOD_H



/*****************************************************************************
   INCLUDES
*****************************************************************************/





/*****************************************************************************
   PROVIDED CONSTANTS
*****************************************************************************/

#define K_SR_OK		0 /* Execution Ok! */
#define K_SR_ERROR	1 /* Generic error */


/* SMCS Error codes */
/* TRS_CTRL register has not the nominal configuration value */
#define K_SR_ERROR_SMCS_TRS_CTRL_NOTNOM		0x101
/* Stop link has been attempted and NULL tokens have not been received */
#define K_SR_ERROR_SMCS_NULLNOTRECEIVED		0x111
/* Start channel has been attempted (either as master or slave) unsuccessfully */
#define K_SR_ERROR_SMCS_LINKNOTCONNECTED	0x121
/* Current Rx Address smaller than Start Rx Address, unable to obtain received data size */
#define K_SR_ERROR_SMCS_BADRXDATASIZE		0x131


/* EDAC Error codes */
/* EDAC configuration is unknown */
#define K_SR_ERROR_EDAC_UNKNOWNCONFIG		0x201
/* PM address does not fall in expected PRAM or EEPROM banks */
#define K_SR_ERROR_EDAC_NOTPRAMNOTEEPROM	0x211


/* DAU Error Codes */
/* Start Acq or read data has been attempted while DAU is still busy */
#define K_SR_ERROR_DAU_DAUBUSY				0x301
/* Start Acq has been attempted and FIFO is not empty */
#define K_SR_ERROR_DAU_FIFONOTEMPTY			0x311
/* During data reading an incorrect number of entries have been found */
#define K_SR_ERROR_DAU_BADDATA				0x321


/* MILSTD-1553 Error Codes */
/* The end of stack was reached while searching the next stack entry */
#define K_SR_ERROR_MILSTD_END_OF_STACK		0x401  



/*****************************************************************************
   PROVIDED TYPES
*****************************************************************************/


/*****************************************************************************
   PROVIDED VARIABLES
*****************************************************************************/


/*****************************************************************************
   DECLARATION OF PROVIDED FUNCTIONS
*****************************************************************************/



#endif  /* L_ERRCOD_H */

