/***************************************************************************************
* File name: pacs_srt.c
* Version.Revision: HPSPU-HLSW-OB-V12R0
*
* Purpose:  It contains the main sorting functions.
*
* Public Functions: None 
*
* Private Functions: None
*
* Description:	This file contains the used sorting functions in the first step (Burrow
                Wheeler transform).
*
* Creation date & author: 28.10.2004 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 18.01.2006 by Ahmed Nabil Belbachir
****************************************************************************************/

/* The Header Files 
===================================================================================================*/
#include "genspu.h"
#include "spulib.h"

extern void pacs_srt(u4 *inout, u4 length, u4 *indexlast, unsigned int order);
void mem_alloc(void);

#define BITSSAMEBLOCK 16
#define BLOCKSIZE (1<<BITSSAMEBLOCK)
#define BLOCKSIZE1 (BLOCKSIZE>>2)
#define BLOCKMASK (BLOCKSIZE-1)

typedef struct p_block ptrblock;
extern u4 *mem_1, *mem_2, *tmp_buf;


struct p_block{
	u4 msbytes[BLOCKSIZE>>1];
	u4 lsbyte[BLOCKSIZE>>1];
	ptrblock *nextfree;
};

typedef struct {
	ptrblock **index;		// index to blocks used in current sort
	ptrblock **oldindex;	// spare mem for alternate index
	ptrblock *freelist;
	ptrblock *block;
	ptrblock *spare[18];
	u4 nrblocks;
} ptrstruct;

ptrstruct globalptr;
int globalinit = 0;
ptrstruct p_p;

static void allocptrs(u4 length, ptrstruct *p)
{	u4 i;
	p->nrblocks = (length+BLOCKSIZE-1)>>BITSSAMEBLOCK; ///BLOCKSIZE;
	p->index = (ptrblock **)(tmp_buf);
	p->oldindex = (ptrblock **)(tmp_buf+0x2000);
	p->block = (ptrblock *)(tmp_buf+0x4000);
	p->freelist = NULLO;
	for(i=0; i<18; i++)
		p->spare[i] = NULLO;
	for (i=0; i<p->nrblocks; i++)
		p->index[i] = p->block + i;
}

void extraspare(ptrstruct *p, int blocks)
{	int i;

	p->spare[0] = (ptrblock*) mem_2;//switchbuf;
	p->spare[0]->nextfree = p->freelist;
	p->freelist = p->spare[0];
	for(i=1; i<blocks; i++)
		p->freelist[i-1].nextfree = p->freelist + i;
	p->freelist[blocks-1].nextfree = NULLO;
}

void allocspareptrs(u4 length, ptrstruct *p)
{
	length = (length>>BITSSAMEBLOCK) + 1;
	if (length>256) length = 256;
	extraspare(p,length);
}

void setptr(ptrstruct *p, u4 i, u4 ptr)
{	ptrblock *tmp;

	tmp = p->index[i>>BITSSAMEBLOCK];
	if (tmp==NULLO)
	{	
		if (p->freelist == NULLO)
			extraspare(p,16);
		tmp = p->index[i>>BITSSAMEBLOCK] = p->freelist;
		p->freelist = p->freelist->nextfree;
	}
	i &= BLOCKMASK;
	tmp->msbytes[i] = ptr>>8;
	tmp->lsbyte[i] = ptr & 0xff;
}

void setptr1(ptrstruct *p, u4 i, u4 ptr)
{	ptrblock *tmp;


	tmp = p->index[i>>BITSSAMEBLOCK];
	i &= BLOCKMASK;
	tmp->msbytes[i] = ptr>>8;
	tmp->lsbyte[i] = ptr & 0xff;
}
static void sortorder2(ptrstruct *p, u4 *in, u4 length,
					   u4 *counts, u4 offset, u4 *indexlast)
{	u4 i, *o2counts, sum, loffs, i1;
	u4 context;
/* Clear Memory */

	for(i=0;i<256;i++)
		counts[i]= 0;;
	/* Temporary buffer for table */
	o2counts = mem_1;
	for(i=0;i<0x10000;i++)
		o2counts[i]= 0;

/* Context Sorting and freq table*/
   context = in[length-1]<<8;
	for(i=0; i<length; i++)
	{	
		context = context>>8 | (in[i])<<8;
		counts[in[i]]++;
		o2counts[context]++;
	}

	sum = length;
	for (i=0x10000; i--; )
	{	sum -= o2counts[i];
		o2counts[i] = sum;
	}

	sum = length;
	for (i=0x100; i--; )
	{	sum -= counts[i];
		counts[i] = sum;
	}

	context = in[length-offset]<<8 | in[length-offset-1];
	if (context == 0xffff)
		*indexlast = length-1;
	else
		*indexlast = o2counts[context+1]-1;
	offset--;
    loffs =length -offset;
	for(i=0; i<offset; i++)
	{	
		i1=i+length;
		in[i1] = in[i];
		context = context>>8 | (in[i+loffs])<<8;
		setptr1(p,o2counts[context]++,i1);
	}

	for(i=offset; i<length; i++)
	{
		context = context>>8 | (in[i-offset])<<8;
		setptr1(p,o2counts[context]++,i);
	}
}

void incsortorder(ptrstruct *p, u4 *in, u4 length,
						 u4 *counts, int offset, u4 *indexlast)
{	u4 i, block, ct[256];
	ptrblock *curblock;
	u4 ch=0;
	{
		ptrblock **swap=p->index;
		p->index = p->oldindex;
		p->oldindex = swap;
	}
	for(i=0;i<p->nrblocks;i++)
		p->index[i]= 0;
	MemCopyDmToDm(counts, ct, 256);
	block = 0;
	curblock = p->oldindex[block];
	for (i=0; i<=*indexlast; i++)
	{	
		u4 tmp = (curblock->msbytes[i])<<8 | curblock->lsbyte[i];
		ch = in[tmp-offset];
		setptr(p,ct[ch],tmp);
		ct[ch]++;
		if (i==BLOCKMASK && block!=p->nrblocks-1)		//last ptr in block
		{	curblock->nextfree = p->freelist;
			p->freelist = curblock;
			block++;
			curblock = p->oldindex[block];
		}
	}
	*indexlast = ct[ch]-1;
	for ( ; i<length; i++)
	{	
		u4 tmp = (curblock->msbytes[i])<<8 | curblock->lsbyte[i];
		ch = in[tmp-offset];
		setptr1(p,ct[ch],tmp);
		ct[ch]++;
		if (i==BLOCKMASK && block<p->nrblocks-1)		//last ptr in block
		{	curblock->nextfree = p->freelist;
			p->freelist = curblock;
			block++;
			curblock = p->oldindex[block];
		}
	}
	curblock->nextfree = p->freelist;
	p->freelist = curblock;
}

static void finishsort(ptrstruct *p, u4 *in, u4 length,
						 u4 *ct, u4 *indexlast)
{	u4 i, block;
	ptrblock *curblock;
	u4 ch=0;
	{
		ptrblock **swap=p->index; p->index = p->oldindex; p->oldindex = swap;
	}
	for(i=0;i<p->nrblocks;i++)
		p->index[i]= 0;
	block = 0;
	curblock = p->oldindex[block];
	for (i=0; i<=*indexlast; i++)
	{	
		u4 tmp = (curblock->msbytes[i])<<8 | curblock->lsbyte[i];
		ch = in[tmp-1];
		setptr(p,ct[ch],in[tmp]);
		ct[ch]++;
		if (i==BLOCKMASK && block!=p->nrblocks-1)		//last ptr in block
		{	
			curblock->nextfree = p->freelist;
		p->freelist = curblock;
			block++;
			curblock = p->oldindex[block];
		}
	}

	*indexlast = ct[ch]-1;
	for ( ; i<length; i++)
	{	
		u4 tmp = (curblock->msbytes[i])<<8 | curblock->lsbyte[i];
		ch = in[tmp-1];
		setptr1(p,ct[ch],in[tmp]);
		ct[ch]++;
		if (i==BLOCKMASK && block!=p->nrblocks-1)		//last ptr in block
		{	
			curblock->nextfree = p->freelist;
			p->freelist = curblock;
			block++;
			curblock = p->oldindex[block];
		}
	}
	curblock->nextfree = p->freelist;
	p->freelist = curblock;

	
	for (i=0; i<p->nrblocks-1; i++)
		MemCopyDmToDm(p->index[i]->lsbyte, in+i*BLOCKSIZE, BLOCKSIZE1);
	i= p->nrblocks - 1;
	MemCopyDmToDm(p->index[i]->lsbyte, in+i*BLOCKSIZE, length-(i*BLOCKSIZE1));
}

// Memory Allocation

void mem_alloc(void)
{
/* allocate memory */
	allocptrs(0xFFFF, &p_p);
/* Allocate memory for spare pointers*/	
	allocspareptrs(0xFFFF, &p_p);
}

/* The main function*/
void pacs_srt(u4 *inout, u4 length, u4 *indexlast, unsigned int order)
{	
	u4 i;
	u4 counts[256];

	mem_alloc();

/* Sorting the data at nth order*/
	sortorder2(&p_p, inout, length, counts, order, indexlast);


/* Sorting according to different orders */
	for (i=order-2; i>1; i--)
		incsortorder(&p_p, inout, length, counts, i, indexlast);

/* flushsorting */
	finishsort(&p_p, inout, length, counts, indexlast);
}
