/***************************************************************************************
* File name: qsmodel.c
* Version.Revision: HPSPU-HLSW-OB-V12R0
*
* Purpose:  It contains the functions for the quick sort model.
*
* Public Functions: None 
*
* Private Functions: None
*
* Description:	This file contains the used quick sort functions.
*
* Creation date & author: 28.10.2004 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 18.01.2006 by Ahmed Nabil Belbachir
****************************************************************************************/

/* The Header Files 
===================================================================================================*/

#include "globals.h"
#include "qsmodel.h"
#define u2 unsigned int

/* default tablesize 1<<TBLSHIFT */
#define TBLSHIFT 7
extern u4 *mem_5,*mem_6, *mem_8, *mem_9;

/* rescale frequency counts */
void dorescale( qsmodel *m)
{   int i, cf, missing;
    if (m->nextleft)  /* we have some more before actual rescaling */
    {   m->incr++;
        m->left = m->nextleft;
        m->nextleft = 0;
        return;
    }
    if (m->rescale < m->targetrescale)  /* double rescale interval if needed */
    {   m->rescale <<= 1;
        if (m->rescale > m->targetrescale)
            m->rescale = m->targetrescale;
    }
    cf = missing = m->cf[m->n];  /* do actual rescaling */
    for(i=m->n-1; i; i--)
    {   int tmp = m->newf[i];
        cf -= tmp;
        m->cf[i] = cf;
        tmp = tmp>>1 | 1;
        missing -= tmp;
        m->newf[i] = tmp;
    }
	m->newf[0] = m->newf[0]>>1 | 1;
    missing -= m->newf[0];
    m->incr = missing/m->rescale; 
    m->nextleft = missing % m->rescale;
    m->left = m->rescale - m->nextleft;
    if (m->search != NULLO)
    {   i=m->n;
        while (i)
        {   int start, end;
            end = (m->cf[i]-1) >> m->searchshift;
            i--;
            start = m->cf[i] >> m->searchshift;
            while (start<=end)
            {   m->search[start] = i;
                start++;
            }
        }
    }
}
/* initialisation of qsmodel                           */

void initqsmodel( qsmodel *m, int n, int lg_totf, int rescale, int *init)
{
	m->n = n;
    m->targetrescale = rescale;
    m->searchshift = lg_totf - TBLSHIFT;
    if (m->searchshift < 0)
        m->searchshift = 0;
    m->cf = mem_5;
    m->newf = mem_6;
    m->cf[n] = 1<<lg_totf;
    m->cf[0] = 0;
    m->search = NULLO;
    resetqsmodel(m, init);
}

void initqsmodel1( qsmodel *m, int n, int lg_totf, int rescale, int *init,int index)
{
	m->n = n;
    m->targetrescale = rescale;
    m->searchshift = lg_totf - TBLSHIFT;
    if (m->searchshift < 0)
        m->searchshift = 0;
    m->cf = mem_8+(index*0x20);
    m->newf = mem_9+(index*0x20);
    m->cf[n] = 1<<lg_totf;
    m->cf[0] = 0;
    m->search = NULLO;
    resetqsmodel(m, init);
}

/* reinitialisation of qsmodel                         */
void resetqsmodel( qsmodel *m, int *init)
{   int i, end, initval;
    m->rescale = m->n>>4 | 2;
    m->nextleft = 0;
    if (init == NULLO)
    {   initval = roldiv(m->cf[m->n], m->n);
        end = m->cf[m->n] % m->n;
        for (i=0; i<end; i++)
            m->newf[i] = initval+1;
        for (; i<m->n; i++)
            m->newf[i] = initval;
    } else
        for(i=0; i<m->n; i++)
            m->newf[i] = init[i];
    dorescale(m);
}

/* retrieval of estimated frequencies for a symbol     */
void qsgetfreq( qsmodel *m, int sym, int sym1, int *sy_f, int *lt_f )
{   *sy_f = m->cf[sym1] - (*lt_f = m->cf[sym]);
}	

/* update model                                        */
void qsupdate( qsmodel *m, int sym )
{
//	E6++;
	if (m->left <= 0)
        dorescale(m);
    m->left--;
    m->newf[sym] += m->incr;
}
