/***************************************************************************************
* File name: Dorada.s
* Version.Revision: HPSPU-HLSW-OB-V3R0
*
* Purpose:  Install an interrupt on DSP IRQ2 for SMCS handling.
*
* Public Functions: None 
*
* Private Functions: SmcsIsr
*
* Description:	
*
* Creation date & author: 21.06.2002 by Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V3R0 24.01.2003 by Ahmed Nabil Belbachir
****************************************************************************************/

/* The Header Files and the Routine Declaration
****************************************************************************************/

#include <def21020.h>      /* STATUS REGISTER BIT DEFINITIONS FOR ADSP-21020 */
#include "macro.h"         /* Macro's for System Kernel (Full Path)*/
#define INCLUDE_DEFINES_ONLY
#include "C:\Virtuoso\Adi21020\sources\current\spuasw.h"
#include "C:\Virtuoso\Adi21020\sources\current\l_hwmap.h"
.segment /pm seg_pmco;     /* Program Code Segment */
.global _SmcsIsr;          /* Declaration of the Assembler Function */
.global _dmpsc_isr;
.global _pmpsc_isr;

/***************************************************************************************/
/* Used External Arguments and Global Variables 
****************************************************************************************/

.extern _K_Args;          /* Channel for the Kernel. ISR Stack Argument */
.extern _K_ArgsP;		  /* Channel for the Kernel. Pointer to ISR Stack Argument */
.extern _SmcsEvent;       /* Event on Kernel Channel */
.extern _MemoryScrubbingParams; /* The container for all memory scrubbing parameters */

/***************************************************************************************/
/* Begin of the Function SmcsIsr 
****************************************************************************************/

_SmcsIsr:

START_ISR; 
/* Pushes Event on Kernel Channel */

	i4 = dm(_K_ArgsP);
	r2 = dm(_SmcsEvent); 
	PRHI_STACK_PSH;

/* End of Interrupt Routine. Control Transfered to System Kernel */
ENDISR1;


/****************************************************************************************
ISR : _dmpsc_isr

AUTHOR : AMazy

USE :
This is the DMPSC interrupt routine.
****************************************************************************************/

 _dmpsc_isr:

	START_ISR; //save MODE1, ASTAT, r0, r1, r2, r4, r8, r12, i4, i12

	/***********************/
	/*Handle EDAC interrupt*/
	/***********************/

	r0 = dm(K_DMADD_DMPSC_INTP);			/* read DMPSC interrupt register*/

	btst r0 by K_BPOS_PSCINT_SF;			/* check if it is a single bit failure*/
	if sz jump (dmpsc_test_df);				/* if not, jumpt to test if it is a double bit failure*/

	r1 = dm(K_DMADD_DMPSC_SFADD);			/* read the address where the sf has been detected*/
	i4 = r1;								/* load this address in i4*/
	r2 = dm(i4, 0);							/* read the content of this address, the content will be corrected by EDAC*/
	dm(i4, 0) = r2;							/* write the corrected content at this address*/

	/* store last sf error */
	i4 = dm(_MemoryScrubbingParams);		/* load the address of gpMemoryScrubbing into i4 */
	r4 = dm(2, i4);							/* the index where to store the next SF error address */
	r2 = K_DM_SF_SCRUB_RES_START_ADDR;		/* the base address of the array containing the last error addresses */ 
	r2 = r2 + r4;							/* the offset between gpMemoryScrubbing and the place where to store the error address */
	m4 = r2;
	dm(m4, i4) = r1;						/* store the faulty address */

	r1 = dm(6, i4);							/* increment SF counter */
	r1 = r1 + 1;
	dm(6, i4) = r1;

	r4 = r4 + 1;
	r2 = MEMORY_SCRUBBING_BUFFER_SIZE ;
	comp(r4, r2);
	if lt jump(dmpsc_test_sf_2);
	r4 = 0;

dmpsc_test_sf_2:
	dm(2, i4) = r4;

	r4 = dm(K_DMADD_DMPSC_INTRST);			/* reset the interrupt */
	r4 = bset r4 by K_BPOS_PSCINT_SF;
	dm(K_DMADD_DMPSC_INTRST) = r4;

dmpsc_test_df:

	btst r0 by K_BPOS_PSCINT_DF;			/* check if it is a double bit failure*/
	if sz jump (dmpsc_exit_isr);			

	r1 = dm(K_DMADD_DMPSC_DFADD);			/* read the address where the sf has been detected*/

	/* store last sf error */
	i4 = dm(_MemoryScrubbingParams);		/* load the address of gpMemoryScrubbing into i4 */
	r4 = dm(4, i4);							/* the index where to store the next DF error address */
	r2 = K_DM_DF_SCRUB_RES_START_ADDR;		/* the base address of the array containing the last error addresses */ 
	r2 = r2 + r4;							/* the offset between gpMemoryScrubbing and the place where to store the error address */
	m4 = r2;
	dm(m4, i4) = r1;						/* store the faulty address */

	r1 = dm(7, i4);							/* increment DF counter */
	r1 = r1 + 1;
	dm(7, i4) = r1;

	r4 = r4 + 1;
	r2 = MEMORY_SCRUBBING_BUFFER_SIZE;
	comp(r4, r2);
	if lt jump(dmpsc_test_df_2);
	r4 = 0;

dmpsc_test_df_2:
	dm(4, i4) = r4;

	r4 = dm(K_DMADD_DMPSC_INTRST);			/* reset the interrupt */	
	r4 = bset r4 by K_BPOS_PSCINT_SF;
	r4 = bset r4 by K_BPOS_PSCINT_DF;
	dm(K_DMADD_DMPSC_INTRST) = r4;

dmpsc_exit_isr :
	ENDISR1;


/****************************************************************************************
ISR : _pmpsc_isr

AUTHOR : AMazy

USE :
This is the PMPSC EDAC interrupt routine

****************************************************************************************/


 _pmpsc_isr:

	START_ISR; //save MODE1, ASTAT, r0, r1, r2, r4, r8, r12, i4, i12
	/***********************/
	/*Handle EDAC interrupt*/
	/***********************/

	r0 = pm(K_PMADD_PMPSC_INTP);			/* read PMPSC interrupt register*/

	btst r0 by K_BPOS_PSCINT_SF;			/* check if it is a single bit failure*/
	if sz jump (pmpsc_test_df);				/* if not, jumpt to test if it is a double bit failure*/

	r1 = pm(K_PMADD_PMPSC_SFADD);			/* read the address where the sf has been detected*/
	i12 = r1;								/* load this address in i12*/
	r2 = pm(i12, 0);						/* read the content of this address, the content will be corrected by EDAC*/
	pm(i12, 0) = r2;						/* write the corrected content at this address*/

	/* store last sf error */
	i4 = dm(_MemoryScrubbingParams);		/* load the address of gpMemoryScrubbing into i4 */
	r4 = dm(3, i4);							/* the index where to store the next SF error address */
	r2 = K_PM_SF_SCRUB_RES_START_ADDR;		/* the base address of the array containing the last error addresses */ 
	r2 = r2 + r4;							/* the offset between gpMemoryScrubbing and the place where to store the error address */
	m4 = r2;
	dm(m4, i4) = r1;						/* store the faulty address */

	r1 = dm(6, i4);							/* increment SF counter */
	r1 = r1 + 1;
	dm(6, i4) = r1;

	r4 = r4 + 1;
	r2 = MEMORY_SCRUBBING_BUFFER_SIZE ;
	comp(r4, r2);
	if lt jump(pmpsc_test_sf_2);
	r4 = 0;

pmpsc_test_sf_2:
	dm(3, i4) = r4;

pmpsc_test_df:

	btst r0 by K_BPOS_PSCINT_DF;			/* check if it is a double bit failure*/
	if sz jump (exit_pmpsc_isr);			

	r1 = pm(K_PMADD_PMPSC_DFADD);			/* read the address where the sf has been detected*/

	/* store last sf error */
	i4 = dm(_MemoryScrubbingParams);		/* load the address of gpMemoryScrubbing into i4 */
	r4 = dm(5, i4);							/* the index where to store the next DF error address */
	r2 = K_PM_DF_SCRUB_RES_START_ADDR;		/* the base address of the array containing the last error addresses */ 
	r2 = r2 + r4;							/* the offset between gpMemoryScrubbing and the place where to store the error address */
	m4 = r2;
	dm(m4, i4) = r1;						/* store the faulty address */

	r1 = dm(7, i4);							/* increment DF counter */
	r1 = r1 + 1;
	dm(7, i4) = r1;

	r4 = r4 + 1;
	r2 = MEMORY_SCRUBBING_BUFFER_SIZE;
	comp(r4, r2);
	if lt jump(pmpsc_test_df_2);
	r4 = 0;

pmpsc_test_df_2:
	dm(5, i4) = r4;

	/*reset the interrupt.  This is done at the end of the routine to avoid infinite triggering of the interrupt in
	case of permanent error on a memory cell.  If there is a permanent error, the interrupt will be raised again when
	we write again the memory location.  Since the interrupt is reset only at the end, we will not catch it.*/

exit_pmpsc_isr:
	r4 = pm(K_PMADD_PMPSC_INTRST);		
	r4 = bset r4 by K_BPOS_PSCINT_SF;
	r4 = bset r4 by K_BPOS_PSCINT_DF;
	pm(K_PMADD_PMPSC_INTRST) = r4;

	ENDISR1;

.endseg;

/* End of the Function SmcsIsr 
********************************************************************************/

