/***************************************************************************************
* File name: Spec1_2.c
* Purpose: Performs Data Compression for Two Spectroscopic Modes.
*
* Description:	This Program Performs Data Compression in Spectroscopy for the
*				Spec1 or Spec2 Modes Depending on the Received Compression Parameter.
*				Spec1 is the Default Compression Mode
*				Spec2 is Double the Compression Mode
***************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#include "time.h"
#include "stdio.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"	

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/
extern void RampFit(u4 readoutsPerRamps, int PixelsForRounding, int FramesForRounding, u4 CompressionMode);
extern void LosslessCompressionForSpectro(COMPRESSION_MODE mode);
/*=================================================================================================*/


/*
This function implements the reduction and compression for default and double mode
*/
void SpectroReductionAndCompressionMode(u4 nbFrames, COMPRESSION_MODE mode)
{
	/*=============================================================================================*/
	/* Be Sure That Number of RC is Compatible with Number of DX*/
	if (RawChannelSelectionSpectro.NbSelectedRawChannels > DxsSpectro.NbSelectedPixels) 
		RawChannelSelectionSpectro.NbSelectedRawChannels = DxsSpectro.NbSelectedPixels;
	
	/* DMC Header Extraction and Compression */
	DmcHeaderCompression(nbFrames, K_SPECTRO_HEADER_SIZE, mode);
	
	/* Detector Selection */
	Dxs(nbFrames * K_SPECTRO_FRAME_SIZE_IN_WORDS, &DxsSpectro, nbFrames, K_CBUF_SCIENCE_SPECTRO, 0, mode);
	
	/* preprocessing of the data */
	PreProcessing(mode.DetectorType);
	
	/* Robust Averaging */
	RampFit(mode.ReadoutsPerRamps, DxsSpectro.NbSelectedPixels, nbFrames, mode.CompressionMode);
	
	/* Data Sorting and Reordering Using the Position of a Sample in a Ramp */
	TSReductionForSpectro(mode);
	
	if (CompressionSoftwareRunning == K_YES) //check that the compression software has not been stopped in the meanwhile
	{
		/* Lossless Coding */
		LosslessCompressionForSpectro(mode);
		
		/*Look now if Raw Channels were selected for further llc */
		if(RawChannels.SizeInWords != 0)
		{
			CompressRawChannels(&RawChannels, mode.DetectorType); 
		}

		/* Write the Compressed Data in the Output Buffer*/
		BuildCompressedEntity(mode);
	}
}
