/***************************************************************************************
* File name: Spec4.c
* Purpose: Performs Data Compression for the 4th Spectroscopic Mode.
*
* Description:	This Program Performs Data Compression in Spectroscopy for the 
*				SpectroTransparentMode Mode According to the Received Compression Parameter.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"	

/*
In transparent mode, data are not compressed (DMC header is compressed).
There is a limitation on the number of selected pixels
*/
void SpectroTransparentMode(u4 nbFrames, COMPRESSION_MODE mode)
{
	/*=============================================================================================*/
	/* Reset of the HK Parameters of Compression Information. These HK are Only used While Reduction
	===============================================================================================*/

	Result.NbMaintainedRamps = 0;
	Result.SaturationFlag=0;
	Result.GlitchCounter = 0;
	Result.NbIntegratedRamps = 0;

	/*=============================================================================================*/

	/* DMC Header Extraction and Compression */
	DmcHeaderCompression(nbFrames, K_SPECTRO_HEADER_SIZE, mode);
  
	/* Detector Selection */
	Dxs(nbFrames * K_SPECTRO_FRAME_SIZE_IN_WORDS, &DxsSpectro, nbFrames, K_CBUF_SCIENCE_SPECTRO, 0, mode);

	CompressedDataBuffer.Data = ProcBuff + 1;
	CompressedDataBuffer.SizeInWords = ProcBuff[0];

	/* Write the Compressed Data in the Output Buffer*/
	BuildCompressedEntity(mode);
}
