/*****************************************************************************************
* File name: spuAsw.h.c
* Description:	This Header File Contains the Compression Parameters, the Memory map and
*				the Data Packet Structures Needed by the Compression Software.
*****************************************************************************************/

#ifndef _spuAsw_H
#define _spuAsw_H

#ifndef INCLUDE_DEFINES_ONLY
#include "genspu.h"
#endif INCLUDE_DEFINES_ONLY

/*===========================================================================================================*/
/* General Definitions 
=============================================================================================================*/

#define K_SCIENCE_HEADER	 0x00890000     /* Science Data Header */

#ifndef SIMULATOR
#define PramBuffer	 0x20000        /* Buffer Transmission Mode Data are Stored in this PRAM Address */
#endif

#define K_SCIENCE_BLOCK_SIZE_IN_WORDS       250			/* Science Data Block (TM Packet) Size in 4-Byte Words*/
#define K_MAX_SIZE_FOR_LLC			        0x7800			/* Maximum Data Size Which Could be Compressed LossLess*/

#define K_HEADER_SIZE	13

/* memory scrubbing definitions*/
#define MEMORY_SCRUBBING_BUFFER_SIZE	128

#define K_DM_SF_SCRUB_RES_START_ADDR	0x4027FE00	/* Start Address of the result buffer for DM scrubbing */
#define K_DM_SF_SCRUB_RES_END_ADDR		0x4027FE7F	
#define K_DM_DF_SCRUB_RES_START_ADDR	0x4027FE80	
#define K_DM_DF_SCRUB_RES_END_ADDR		0x4027FEFF	
#define K_PM_SF_SCRUB_RES_START_ADDR	0x4027FF00	
#define K_PM_SF_SCRUB_RES_END_ADDR		0x4027FF7F	
#define K_PM_DF_SCRUB_RES_START_ADDR	0x4027FF80	
#define K_PM_DF_SCRUB_RES_END_ADDR		0x4027FFFF	



/* Definitions for Spectroscopy */

#define K_SPECTRO 1
#define	K_NB_PIXELS_IN_SPECTRO_ARRAY	468						/* Number of Detectors per Spectro's Array (26x18) */
#define K_NB_USEFUL_PIXELS_IN_SPECTRO_ARRAY	450					/* Number of useful detectors per spectro array (25x18) */
#define K_SPECTRO_FRAME_SIZE_IN_WORDS	234						/* This is Frame Size in 4Bytes Word (26x18/2) */
#define K_NB_SPECTRO_FRAMES_IN_2SEC_BUF	512		                /* Number of Frames per Array in Two Seconds Buffer */
#define K_SPECTRO_HEADER_SIZE	K_HEADER_SIZE								/* size (in words) of a dmc spectro header */
#define K_SPECTRO_DATA_2SEC_BUF_SIZE_IN_WORDS	K_NB_SPECTRO_FRAMES_IN_2SEC_BUF * K_SPECTRO_FRAME_SIZE_IN_WORDS		/* Size of Science Data Buffer in 4-Byte Words (2 second Buffer for 26x18 pixels) */ 
#define	K_SPECTRO_HEADER_2SEC_BUF_SIZE_IN_WORDS K_SPECTRO_HEADER_SIZE* K_NB_SPECTRO_FRAMES_IN_2SEC_BUF				/* Size of DMC Header in 4-Byte Words (2 seconds Buffer) */ 
#define K_SPECTRO_MINIMUM_NB_FRAMES_TO_START_PROCESSING 32  	/* The Minimum Nb of Frames Required in Spectroscopy Buffer to Start Processing */
#define K_LEAST_SQUARE_ALGO 0									/* The Least Squares Algorithm */
#define K_MEAN_ALGO	  	  1										/* The Mean Algorithm */

/* Definitions for Photometry */
#define K_PHOTO 2
#define K_NB_PIXELS_IN_PHOTO_BLOCK 512						/* Number of Detectors per Two Sub-Arrays of Bolometers */
#define K_NB_PIXELS_IN_BLUE_BOLOMETERS K_NB_PIXELS_IN_PHOTO_BLOCK*4		/* Number of Detectors in the blue Bolometers*/
#define K_NB_PIXELS_IN_PHOTO_ARRAY 256						/* Data Frame Size in 4-Byte Words (256 Words for Photometry Frame
																		and 234 Words for Spectroscopy Frame)*/
#define	K_NB_PHOTO_READOUTS_IN_3SEC_BUF	120
#define K_NB_PHOTO_FRAMES_IN_3SEC_BUF K_NB_PHOTO_READOUTS_IN_3SEC_BUF * 4					/* Number of Frames per 4 Sub-Arrays */
#define K_PHOTO_HEADER_SIZE	K_HEADER_SIZE								/* size (in words) of a dmc photo header */
#define K_PHOTO_DATA_3SEC_BUF_SIZE_IN_WORDS K_NB_PHOTO_READOUTS_IN_3SEC_BUF * K_NB_PIXELS_IN_PHOTO_ARRAY		/* Size of Science Data Buffer in 4-Byte Words (3 second Buffer for 16x32 pixels) */ 
#define	K_PHOTO_BLUE_HEADER_3SEC_BUF_SIZE_IN_WORDS K_PHOTO_HEADER_SIZE * K_NB_PHOTO_READOUTS_IN_3SEC_BUF			/* Size of DMC Header in 4-Byte Words (3 seconds Buffer). Taking 13 Words out of 16*/ 
#define	K_PHOTO_RED_HEADER_3SEC_BUF_SIZE_IN_WORDS K_PHOTO_HEADER_SIZE * 4 * K_NB_PHOTO_READOUTS_IN_3SEC_BUF			/* Size of DMC Header in 4-Byte Words (12 seconds Buffer). Taking 13 Words out of 16*/ 
#define K_PHOTO_MINIMUM_NB_FRAMES_TO_START_PROCESSING 16								/* The Minimum Nb of Frames Required in Red Photometry Buffer to Start Processing.
															   This Number is 4 for Blue Photometer */

/* Definitions for Arithmetic Encoding */
#define PHOT_MODEL 0
#define SPEC_MODEL 1

#ifndef INCLUDE_DEFINES_ONLY

/*===========================================================================================================*/
/* Initialization of the Addresses for the Circular Buffers 
=============================================================================================================*/

typedef enum
{
	K_CIRC_BUFFER_START_ADDR_DMCH		= 0x000000A0,	/* Start Address of the Input Buffer of the DEC/MEC Header */
	K_CIRC_BUFFER_END_ADDR_DMCH			= 0x000034AF,	/* End Address of the Input Buffer of the DEC/MEC Header */
	K_CIRC_BUFFER_START_ADDR			= 0x00003500,	/* Start Address of the Science Buffer Spectroscopy */
	K_CIRC_BUFFER_END_ADDR				= 0x0003F57F,	/* End Address of the Science Buffer Spectroscopy */
	K_CIRC_BUFFER_END_ADDR1				= 0x0001251F,	/* End Address of the Science Buffer Photometry (Sub-Array 1) */
	K_CIRC_BUFFER_END_ADDR2				= 0x0002153F,	/* End Address of the Science Buffer Photometry (Sub-Array 2) */
	K_CIRC_BUFFER_END_ADDR3				= 0x0003055F,	/* End Address of the Science Buffer Photometry (Sub-Array 3) */
	K_CIRC_BUFFER_START_ADDR_TM			= 0x40270000,	/* Start Address of the TM Packets Buffer */
	K_CIRC_BUFFER_END_ADDR_TM			= 0x4027FDFF,	/* End Address of the TM Packets Buffer*/
	K_MAX_NB_TM_PACKETS     = 37,			/* The Maximum Number of TM Packets Possible To Store in the TM Buffer */
}INPUT_BUFFER;

/*===========================================================================================================*/
/* Declaration of a General DEC/MEC Data Packet 
=============================================================================================================*/

struct	SCIENCE_PACKET					/* The parameters are defined in 4 bytes word */
{
	u4 SpuId	;						/* SPU ID (LWL or SWL) */
	u4 Type;							/* HK Header */
	u4 ObsId;							/* OBS ID */
	u4 BbId;							/* Building Block ID */
	u4 Label;							/* Label */
	u4 Time[2];							/* Time */
	u4 Validity;						/* The Sample Validity */	   	
	u4 ChopperPos;						/* Chopper Position Readback */	
	u4 WheelPos;						/* Wheel Position Readback*/
	
	union
	{
		u4 GratingPos;					/* Grating Position Readback for Spectroscopy */
		u4 BolcStatus;					/* BolC Status for Photometry */
	}Block1;
	
	u4 CurrentReadoutCounter;			/* Current Readout Count in OBT Ticks Since the Last*/
	
	union
	{
		u4 ReadoutsPerRamps;			/* Readouts in Ramp Readback */	
		u4 CurrentReadoutCountInChopperPlateau;		/* Current Readout Count in Chopper Position for Photometry */
	}Block2;
	
	union
	{
		u4 CurrentRampCounter;			/* Current Ramp Count */
	  	u4 DataBlockId;					/* Data Block ID in Photometry */	
	}Block3;
	
	union
	{
		u4 CreControlReadback;			/* CRE Control Readback */
		u4 BolSetupId;					/* Bolometer Setup ID */
	}Block4;
	
	u4 CompressionMode;					/* The Compression Mode */
	u4 Data[K_NB_PIXELS_IN_PHOTO_ARRAY];/* The Data Readouts */
};											

/*===========================================================================================================*/
/* Declaration of Compression Parameters 
=============================================================================================================*/

typedef struct COMPRESSION_MODE			/* The parameters are defined in 4 bytes word */
{
	u4 CompressionMode;					/* Compression Mode ID */
	u4 ObsId;							/* OBS ID */
	u4 ReadoutsPerRamps;				/* Readouts in Ramp Readback */
	u4 PacketIndex;						/* Index of the Compressed Entity*/
	u4 DataBlockId;						/* Data Block ID */
	u4 DetectorType;					/* Type of Detectors (K_SPECTRO or K_PHOTO)*/
	u4 BolcDataType;					/* extracted from BOLC STATUS (0 = S-R, 1 = Test, other values are also valid, see BOLC status definition (note that this variable only has the 3 MSB of the data type)*/
} COMPRESSION_MODE;

/*===========================================================================================================*/
/* Declaration of Packet Ready to Transmit 
=============================================================================================================*/

struct PACKET_READY						/* Transmit Compressed Entity to DPU*/
{									
	u4 Header;							/* Science Packet Header */
	u4 Size;							/* Size of Packet to Transmit */
} ;


/*===========================================================================================================*/
/* Declaration of compression software fifo messages
=============================================================================================================*/
enum
{
	K_COMPRESS_DATA,
};

struct FIFO_CSW_MSG
{
	u4					Message;
	u4					NbFrames;
	COMPRESSION_MODE	Mode;
	u4					Spare;
};

/*===========================================================================================================*/
/* Declaration of Processing Result Packet to Fill HK Packet 
=============================================================================================================*/

struct PROCESSING_RESULT				/* The Results After Compression are Stored Here*/
{
	u4 PacketIndex;						/* Packet Index During Compression*/
	u4 ReductionAlgo;					/* Reduction Algorithm Used */
	u4 SaturationFlag;					/* Saturation Flag */
	u4 GlitchCounter;					/* Glitch Counter Information */
	u4 NbMaintainedRamps;				/* Number of Maintained Ramps/Averages */
	u4 NbIntegratedRamps;				/* Number of Integrated Ramps */
	u4 RawChannelIndex;					/* Raw Channel Index */
	u4 DmcHeaderError;					/* DMC Header Error */
	u4 ObsId;							/* OBSID */
	u4 CompStatus;						/* Compression Status */
	u4 LlcError;						/* LLC write parameter Error */
} ;

/* ReductionAlgo values:
Photometry.
	 0: Mean of Signed quadruples
	 1: Mean of Unsigned quadruples
	 2: Mean of 2 Signed samples
	 3: Mean of 2 Unsigned samples
Spectroscopy.
	 0x10: Least Square with Glitch Detection
	 0x11: Least Square without Glitch Detection
	 0x12: Mean with Glitch Detection
	 0x13: Mean without Glitch Detection
	 0x14: Sample Difference for 2 samples per Subramp
*/

/*===========================================================================================================*/
/* Declaration of ERROR_CODES
=============================================================================================================*/
enum ERROR_CODES
{
	K_DMC_NO_ERROR									= 0x00,
	K_DMC_BUFFER_UNDER_GRANULARITY					= 0x02,
	K_DMC_INVALID_SPU_ID							= 0x04,
	K_DMC_INVALID_BLOCK_ID							= 0x05,
	K_DMC_SPU_ID_INCOMPATIBLE_WITH_DBID				= 0x06,
	K_DMC_INVALID_COMPRESSION_MODE					= 0xAA,
	K_DMC_UNABLE_TO_PROCESS_DATA_FAST_ENOUGH		= 0xDD,	
	K_DMC_DPU_IS_TOO_SLOW							= 0xEE,	
	K_DMC_INVALID_DATA_TYPE							= 0xFF,
};

/*===========================================================================================================*/
/* Declaration of COMPRESSION_STATUS
=============================================================================================================*/
enum COMPRESSION_STATUS
{
	K_COMP_STATUS_CSW_RUNNING						= 0xDDDD0000,
	K_COMP_STATUS_CSW_STOPPED						= 0xFFFF0000,
	K_COMP_STATUS_COMPRESSED_ENTITY_INCOMPLETE		= 0xAAAA0000,
};


/*===========================================================================================================*/

#endif //#ifndef INCLUDE_DEFINES_ONLY
#endif

