/***************************************************************************************
* File name: W_Reset.c
* Purpose:  Acknowledges the Warm Reset Command.
*
* Description:	This Program Acknowledges the Reception of the Warm Reset
*				Command and Performs the Appropriate Action for the HLSW Reset in Case 
*				of Command Consistency.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spulib.h"

void PerformWarmReset(u4 buf[])
{
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	// message to send information to the data_tx task that will send the acknowledge to the DPU
	u4 sid = buf[1] & 0xFFFF;

	/*=============================================================================================*/
	
	if (sid!=0) /* Negative Acknowledgment: Invalid Structure ID*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID ;
	  	fifoTransmitMsg.Block1.NackParameter = sid;
		
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else /* Positive Acknowledgment */		
	{
		fifoTransmitMsg.Header = K_PACK_PERFORM;
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	
		/* Delay to Alow the Transmission of the PACK*/
        KS_TaskSleep(20); 
	 
		/* Reset All the Tasks*/
		KS_SemaSignal(WARM_RESET_SEMA);
	}
}
